package com.zanxiang.game.module.manage.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.zanxiang.game.module.manage.enums.GameServerExcelEnum;
import com.zanxiang.game.module.manage.pojo.params.GameMergeServerParam;
import com.zanxiang.game.module.manage.pojo.params.GameServerAddUpdateParam;
import com.zanxiang.game.module.manage.pojo.params.GameServerAssignParam;
import com.zanxiang.game.module.manage.pojo.params.GameServerListParam;
import com.zanxiang.game.module.manage.pojo.vo.GameServerListVO;
import com.zanxiang.game.module.manage.pojo.vo.GameServerVO;
import com.zanxiang.game.module.mybatis.entity.GameServer;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * @author : lingfeng
 * @time : 2023-08-07
 * @description : 游戏区服
 */
public interface IGameServerService extends IService<GameServer> {

    /**
     * 区服指派
     *
     * @param param : 提交参数
     * @return : 返回更新结果
     */
    Boolean gameServerAssignBatch(GameServerAssignParam param);

    /**
     * 区服excel提交
     *
     * @param multipartFile       文件流
     * @param gameServerExcelEnum 操作类型枚举
     * @param gameId              游戏id
     * @return {@link Boolean}
     */
    Boolean gameServerExcelImport(MultipartFile multipartFile, GameServerExcelEnum gameServerExcelEnum, Long gameId);

    /**
     * 列表名字
     *
     * @param gameId     游戏id
     * @param serverName 服务器名称
     * @return {@link List}<{@link String}>
     */
    List<String> listServerId(Long gameId, String serverName);

    /**
     * 添加或更新
     *
     * @param param 参数
     * @return boolean
     */
    boolean addOrUpdate(GameServerAddUpdateParam param);

    /**
     * 列表页面
     *
     * @param param 参数
     * @return {@link IPage}<{@link GameServerListVO}>
     */
    IPage<GameServerListVO> listOfPage(GameServerListParam param);

    /**
     * 删除通过id
     *
     * @param id id
     * @return boolean
     */
    boolean deleteById(Long id);

    /**
     * 合服添加更新
     *
     * @param param 参数
     * @return {@link Boolean}
     */
    Boolean mergeServerAddUpdate(GameMergeServerParam param);

    /**
     * 获取区服列表
     *
     * @param gameId  游戏id
     * @param isMerge
     * @return {@link List}<{@link GameServerVO}>
     */
    List<GameServerVO> getServerList(Long gameId, Boolean isMerge);
}
